<?php

namespace Drupal\hijri_dates\Plugin\Field\FieldFormatter;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\TimestampFormatter;
use Drupal\hijri_dates\Service\HijriDateService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'Hijri Timestamp' formatter.
 *
 * Extends the default timestamp formatter to support Hijri date formats.
 */
class HijriTimestampFormatter extends TimestampFormatter {

  /**
   * The Hijri date service.
   *
   * @var \Drupal\hijri_dates\Service\HijriDateService
   */
  protected $hijriDateService;

  /**
   * Constructs a HijriTimestampFormatter object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Entity\EntityStorageInterface $date_format_storage
   *   The date format storage.
   * @param \Drupal\hijri_dates\Service\HijriDateService $hijri_dates_service
   *   The Hijri date service.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, DateFormatterInterface $date_formatter, EntityStorageInterface $date_format_storage, HijriDateService $hijri_dates_service) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings, $date_formatter, $date_format_storage);
    $this->hijriDateService = $hijri_dates_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('date.formatter'),
      $container->get('entity_type.manager')->getStorage('date_format'),
      $container->get('hijri_dates.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'use_hijri' => FALSE,
      'date_correction' => '',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::settingsForm($form, $form_state);

    $form['use_hijri'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use Hijri calendar'),
      '#description' => $this->t('Convert the selected date format to Hijri calendar.'),
      '#default_value' => $this->getSetting('use_hijri'),
    ];

    $form['date_correction'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Date correction'),
      '#description' => $this->t('Enter a PHP date string modifier (e.g., "+1 day", "-2 days", "+3 months") to adjust Hijri dates based on moon sighting or regional variations. These are standard PHP relative date formats. Leave empty for no correction.'),
      '#default_value' => $this->getSetting('date_correction'),
      '#size' => 20,
      '#maxlength' => 30,
      '#states' => [
        'visible' => [
          ':input[name="fields[' . $this->fieldDefinition->getName() . '][settings_edit_form][settings][use_hijri]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Add validation to check if the selected format is compatible with Hijri dates.
    $form['#validate'][] = [$this, 'validateHijriCompatibility'];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    if ($this->getSetting('use_hijri')) {
      $summary[] = $this->t('Using Hijri calendar');

      // Add date correction to summary if set.
      $date_correction = $this->getSetting('date_correction');
      if (!empty($date_correction)) {
        $summary[] = $this->t('Date correction: @correction', ['@correction' => $date_correction]);
      }
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    if (!$this->getSetting('use_hijri')) {
      return parent::viewElements($items, $langcode);
    }

    $elements = [];
    $date_correction = $this->getSetting('date_correction');
    $format = $this->getSetting('date_format');

    foreach ($items as $delta => $item) {
      $timestamp = $item->value;
      $elements[$delta] = [
        '#markup' => $this->hijriDateService->formatHijri(
          $timestamp,
          $format,
          $date_correction
        ),
        '#cache' => [
          'contexts' => [
            'timezone',
          ],
        ],
      ];
    }

    return $elements;
  }

  /**
   * Validate that the selected format is compatible with Hijri dates.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public function validateHijriCompatibility(array &$form, FormStateInterface $form_state) {
    $use_hijri = $form_state->getValue(['settings', 'use_hijri']);
    if ($use_hijri) {
      $format = $form_state->getValue(['settings', 'date_format']);
      $validation = $this->hijriDateService->validateFormatCompatibility($format);

      if (!$validation['is_valid']) {
        $form_state->setError($form['settings']['use_hijri'], $this->t('@error Please choose a different format or disable Hijri calendar.', ['@error' => $validation['error']]));
      }
    }
  }

}
