<?php

declare(strict_types=1);

namespace Drupal\hijri_dates\Service;

use ArPHP\I18N\Arabic;
use Drupal\Core\Datetime\DateFormatInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Converts Gregorian dates to Hijri dates via the ArPHP library.
 *
 * Usage
 * ------------------------------------------------------------------
 *  • Machine-name   : $svc->formatHijri($ts, 'short');
 *  • Literal pattern: $svc->formatHijri($ts, '!Y-m-d');
 *  • Default        : $svc->formatHijri($ts);               // "medium"
 *  • Corrections    : $svc->formatHijri($ts, 'long', '+1 day');
 */
class HijriDateService {

  use StringTranslationTrait;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The Arabic library instance.
   *
   * @var \ArPHP\I18N\Arabic
   */
  protected $arabic;

  /**
   * Constructs a new HijriDateService object.
   *
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger channel.
   * @param \ArPHP\I18N\Arabic $arabic
   *   The Arabic library instance.
   */
  public function __construct(
    DateFormatterInterface $date_formatter,
    LanguageManagerInterface $language_manager,
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelInterface $logger,
    Arabic $arabic,
  ) {
    $this->dateFormatter = $date_formatter;
    $this->languageManager = $language_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger;
    $this->arabic = $arabic;
  }

  /**
   * Converts a date value to Hijri.
   *
   * @param \DateTimeInterface|int|string $date
   *   Input value (strtotime-parsable string, timestamp, or DateTime).
   * @param string|null $format_id
   *   • Machine-name of a `date_format` entity,
   *   • Literal PHP pattern prefixed with "!",
   *   • NULL → site "medium".
   * @param string|null $date_correction
   *   Relative modifier (“+1 day”, “-2 days”).
   */
  public function formatHijri(
    $date,
    ?string $format_id = NULL,
    ?string $date_correction = NULL,
  ): string {
    $timestamp = $this->toTimestamp($date);
    if ($timestamp === FALSE) {
      return '';
    }

    $this->arabic->setDateMode($this->getAppropriateMode());
    $pattern = $this->choosePattern($format_id);
    $offset  = $this->calculateDateCorrection($timestamp, $date_correction);

    try {
      return $this->arabic->date($pattern, $timestamp, $offset);
    }
    catch (\Throwable $e) {
      $this->logger->error('Hijri conversion failed: @msg', ['@msg' => $e->getMessage()]);
      return date($pattern, $timestamp);
    }
  }

  /**
   * Returns the stored pattern for a date-format entity.
   *
   * @param string $format_id
   *   The date format ID.
   *
   * @return string
   *   The date format pattern.
   */
  public function getDateFormatPattern(string $format_id): string {
    $entity = $this->entityTypeManager
      ->getStorage('date_format')
      ->load($format_id);

    if ($entity instanceof DateFormatInterface) {
      return $entity->getPattern();
    }

    $this->logger->warning('Date format "@id" not found; falling back.', ['@id' => $format_id]);
    return 'Y-m-d H:i';
  }

  /**
   * Validates the compatibility of a date format pattern.
   *
   * @param string $format_id
   *   The date format ID or a literal pattern prefixed with '!'.
   *
   * @return array
   *   An array with the following keys:
   *   - is_valid: bool
   *   - pattern: string
   *   - error: string|null
   */
  public function validateFormatCompatibility(string $format_id): array {
    if (str_starts_with($format_id, '!')) {
      $pattern = substr($format_id, 1);
    }
    else {
      $pattern = $this->getDateFormatPattern($format_id);
    }

    $illegal = ['S', 'T', 'e', 'I', 'O', 'P', 'Z', 'c', 'r', 'U', '\\'];

    foreach ($illegal as $c) {
      if (str_contains($pattern, $c)) {
        return [
          'is_valid' => FALSE,
          'pattern'  => $pattern,
          'error'    => sprintf('Unsupported character "%s".', $c),
        ];
      }
    }
    return ['is_valid' => TRUE, 'pattern' => $pattern, 'error' => NULL];
  }

  /**
   * Gets the appropriate date mode based on language direction.
   *
   * @return int
   *   1 for RTL languages, 8 for LTR languages.
   */
  protected function getAppropriateMode(): int {
    return $this->languageManager->getCurrentLanguage()->getDirection() === 'rtl' ? 1 : 8;
  }

  /**
   * Converts relative date strings to ArPHP offset.
   *
   * @param int $ts
   *   The timestamp.
   * @param string|null $expr
   *   The relative date expression (e.g., "+1 day").
   *
   * @return int
   *   The calculated offset.
   */
  protected function calculateDateCorrection(int $ts, ?string $expr): int {
    if ($expr) {
      $mod = strtotime($expr, $ts);
      if ($mod !== FALSE) {
        return (int) round(($mod - $ts) / 86_400);
      }
      $this->logger->notice('Invalid correction string "@s".', ['@s' => $expr]);
    }
    return $this->arabic->dateCorrection($ts);
  }

  /**
   * Checks if a string could be a PHP date pattern.
   *
   * Used only as final fallback; real literal patterns should be prefixed "!".
   *
   * @param string $s
   *   The string to check.
   *
   * @return bool
   *   TRUE if the string looks like a date pattern, FALSE otherwise.
   */
  protected function isFormatPattern(string $s): bool {
    return (bool) preg_match(
      '/^[dDjlNSwzWFmMntLoYyaABgGhHisueIOPTZcrU\h,:\/\-\.\(\)]+$/u',
      $s
    );
  }

  /**
   * Selects the appropriate date pattern based on input.
   *
   * Pattern-choice priority:
   * - 1. `!literal`
   * - 2. existing entity
   * - 3. heuristic literal (only tokens / punctuation)
   * - 4. "medium"
   *
   * @param string|null $id
   *   The format ID or pattern.
   *
   * @return string
   *   The selected pattern.
   */
  private function choosePattern(?string $id): string {
    if (is_string($id) && str_starts_with($id, '!')) {
      return substr($id, 1);
    }

    $entity = $this->entityTypeManager->getStorage('date_format')->load($id ?? 'medium');
    if ($entity instanceof DateFormatInterface) {
      return $entity->getPattern();
    }

    if ($id && $this->isFormatPattern($id)) {
      return $id;
    }

    return $this->getDateFormatPattern('medium');
  }

  /**
   * Converts mixed input to timestamp with year range validation.
   *
   * @param mixed $value
   *   The value to convert.
   *
   * @return int|false
   *   The timestamp or FALSE if invalid.
   */
  private function toTimestamp($value): int|false {
    if ($value instanceof \DateTimeInterface) {
      return $this->yearOk($value->getTimestamp()) ? $value->getTimestamp() : FALSE;
    }
    if (is_numeric($value) && $this->yearOk((int) $value)) {
      return (int) $value;
    }
    if (is_string($value) && ($ts = strtotime($value)) !== FALSE && $this->yearOk($ts)) {
      return $ts;
    }
    return FALSE;
  }

  /**
   * Validates that a timestamp's year is within allowed range (1900-2100).
   *
   * @param int $ts
   *   The timestamp to check.
   *
   * @return bool
   *   TRUE if the year is valid, FALSE otherwise.
   */
  private function yearOk(int $ts): bool {
    try {
      $year = (int) (new \DateTimeImmutable("@$ts"))
        ->setTimezone(new \DateTimeZone('UTC'))
        ->format('Y');
      return $year >= 1900 && $year <= 2100;
    }
    catch (\Throwable) {
      return FALSE;
    }
  }

}
