<?php

namespace Drupal\hijri_dates\TwigExtension;

use Drupal\hijri_dates\Service\HijriDateService;
use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;
use Twig\TwigFunction;

/**
 * Provides Hijri date Twig filters and functions.
 *
 * This extension adds a 'hijri' filter and function to Twig templates,
 * allowing for easy conversion of dates to the Hijri calendar.
 *
 * Examples:
 * - {{ date|hijri }} - Format a date using default Hijri format
 * - {{ date|hijri('medium') }} - Format using the 'medium' date format
 * - {{ date|hijri('custom', 'd/m/Y') }} - Format using a custom date format
 * - {{ date|hijri(null, '+1 day') }} - Apply date correction
 * - {{ hijri(date) }} - Same as filter, but as a function
 *
 * The language of the output (Arabic or English) is automatically determined
 * based on the current interface language.
 */
class HijriDateTwigExtension extends AbstractExtension {

  /**
   * The Hijri date service.
   *
   * @var \Drupal\hijri_dates\Service\HijriDateService
   */
  protected $hijriDateService;

  /**
   * Constructs a HijriDateTwigExtension object.
   *
   * @param \Drupal\hijri_dates\Service\HijriDateService $hijri_dates_service
   *   The Hijri date service.
   */
  public function __construct(HijriDateService $hijri_dates_service) {
    $this->hijriDateService = $hijri_dates_service;
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return 'hijri_dates.twig_extension';
  }

  /**
   * {@inheritdoc}
   */
  public function getFilters() {
    return [
      new TwigFilter('hijri', [$this, 'formatHijri']),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFunctions() {
    return [
      new TwigFunction('hijri', [$this, 'formatHijri']),
    ];
  }

  /**
   * Formats a date using Hijri calendar.
   *
   * @param mixed $date
   *   A date object or timestamp.
   * @param string|null $format_id
   *   The date format ID to use (e.g., 'medium', 'short', 'long').
   *   If NULL, the default 'medium' format will be used.
   * @param string|null $date_correction
   *   Optional date correction expression (e.g., "+1 day", "-2 days")
   *   to adjust Hijri dates based on moon sighting or regional variations.
   *
   * @return string
   *   The formatted date string in Hijri calendar.
   *
   * @see \Drupal\hijri_dates\Service\HijriDateService::formatHijri()
   */
  public function formatHijri($date, $format_id = NULL, $date_correction = NULL) {
    return $this->hijriDateService->formatHijri($date, $format_id, $date_correction);
  }

}
